#ifndef CARDWIRTH_CONTENT_H_INCLUDED
#define CARDWIRTH_CONTENT_H_INCLUDED

//
// content.h
//
// wanderer7 CardWirth Library Copyright (C) wanderer7 2002
//
//   L̒쌠\Ƃ̃CZXƂς
// cÃ\tgEFARɎgpEEρE
// zzč\܂B
//   L̒쌠\t̃hLgɊ܂߂A
// ̃\tgEFAgݍ񂾃\tgEFAR
// Ezzč\܂B
//   ̃\tgEFA͊Sɖۏ؂łAȂ`
// gpɂĐȂ鑹Qɂ҂͂̐ӂ
// ܂܂B
//
// ȏ
//

//
// CardWirth ́AgroupASK ̃\tgEFAłB
//   GROUP ASK WEBSITE: http://www.ask.sakura.ne.jp/
//

#include <string>
#include <stdexcept>
#include <list>
#include <boost/shared_ptr.hpp>
#include "cwobj.h"

namespace CardWirth
{
	class Content : public CWObject
	{
		typedef std::list< boost::shared_ptr<Content> > contentlist_type;
	public:
		Content() {};
		virtual ~Content() {};
		virtual void dump(std::ostream &out) const throw(IOException) = 0;
		enum {
			Start=0x00, LinkToStart=0x01, StartBattle=0x02, ScenarioClear=0x03,
			GameOver=0x04, ChangeArea=0x05, Message=0x06, ChangeBGM=0x07, 
			ChangeBG=0x08, PlaySE=0x09, Wait=0x0a,	Effect=0x0b,
			Branch_MemberChoice=0x0c, Branch_AbilityCheck=0x0d, Branch_Random=0x0e, Branch_Flag=0x0f, 
			SetFlag=0x10, MultiBranch_Step=0x11, SetStep=0x12, Branch_Presense_Cast=0x13, 
			Branch_Posession_Item=0x14, Branch_Posession_Skill=0x15, Branch_Posession_Info=0x16, Branch_Presense_Beast=0x17,
			Branch_Money=0x18, Branch_Posession_Coupon=0x19, GetCast=0x1a, GetItem=0x1b, 
			GetSkill=0x1c, GetInfo=0x1d, GetBeast=0x1e, GetMoney=0x1f, 
			GetCoupon=0x20, LoseCast=0x21, LoseItem=0x22, LoseSkill=0x23, 
			LoseInfo=0x24, LoseBeast=0x25, LoseMoney=0x26, LoseCoupon=0x27, 
			Serifu=0x28, IncStep=0x29, DecStep=0x2a, InvertFlag=0x2b, 
			BiBranch_Step=0x2c, IncRound=0x2d, Branch_Level=0x2e, Branch_Status=0x2f, 
			Branch_MemberCount=0x30, ShowParty=0x31, HideParty=0x32, BreakEffect=0x33, 
			CallStart=0x34, LinkToPackage=0x35, CallPackage=0x36, Branch_Area=0x37, 
			Branch_Battle=0x38, Branch_ClearedScenario=0x39, AddClearedScenario=0x3a, 
			RemoveClearedScenaio=0x3b, Branch_Gossip=0x3c, AddGossip=0x3d, RemoveGossip=0x3e, Branch_BattleResult=0x3f,
			RedrawScreen=0x40, FlagCheck=0x41
		};
	protected:
		void dump(std::ostream &out, int depth) const throw (IOException);
		virtual void dumpsingle(std::ostream &out) const throw (IOException) = 0;
		void read(std::istream &in) throw (IOException);
		int type;
		std::string label;
		contentlist_type childs;
	};

	class ContentReader {
	public:
		Content *read(std::istream &in) throw(IOException);
	};

#define DECLARE_COMMONFUNC(Name) \
	public: \
		Content_##Name() { init(); }; \
		Content_##Name(std::istream &in) throw (IOException) { read(in); }; \
		void read(std::istream &in) throw(IOException); \
		virtual ~Content_##Name() {}; \
		virtual void dump(std::ostream &out) const throw(IOException) { Content::dump(out, 0); }; \
	protected: \
		virtual void dumpsingle(std::ostream &out) const throw (IOException);

	class Content_Start : public Content {
		DECLARE_COMMONFUNC(Start);
	public:
		void init() {};
	};

	class Content_Terminal : public Content {
	public:
		Content_Terminal() {}; \
		virtual ~Content_Terminal() {}; \
		virtual void dump(std::ostream &out) const throw(std::runtime_error) = 0; \
	};

	class Content_Branch : public Content {
	public:
		Content_Branch() {}; \
		virtual ~Content_Branch() {}; \
		virtual void dump(std::ostream &out) const throw(IOException) = 0; \
	};

	class Content_Normal : public Content {
	public:
		Content_Normal() {}; \
		virtual ~Content_Normal() {}; \
		virtual void dump(std::ostream &out) const throw(IOException) = 0; \
	};

	class Content_LinkToStart : public Content_Terminal {
		DECLARE_COMMONFUNC(LinkToStart);
	public:
		void init() {};
		const std::string &getStartName() { return startName; };
	protected:
		std::string startName;
	};

	class Content_StartBattle : public Content_Terminal {
		DECLARE_COMMONFUNC(StartBattle);
	public:
		void init() { battleNo = 0; };
		int getBattleNo() { return battleNo; };
	protected:
		int battleNo;
	};

	class Content_ScenarioClear : public Content_Terminal {
		DECLARE_COMMONFUNC(ScenarioClear);
	public:
		void init() {};
		bool &ifMarkZumiin() { return zumiin; };
	protected:
		bool zumiin;
	};

	class Content_GameOver : public Content_Terminal {
		DECLARE_COMMONFUNC(GameOver);
	public:
		void init() {};
	};

	class Content_ChangeArea : public Content_Terminal {
		DECLARE_COMMONFUNC(ChangeArea);
	public:
		void init() {};
		int getAreaNo() { return areano; };
	protected:
		int areano;
	};

	class Content_Message : public Content_Normal {
		DECLARE_COMMONFUNC(Message);
	public:
		void init() {};
		const std::string &getImageFileName() { return imageFile; };
		const std::string &getMessage() { return message; };
	protected:
		std::string imageFile;
		std::string message;
	};

	class Content_ChangeBGM : public Content_Normal {
		DECLARE_COMMONFUNC(ChangeBGM);
	public:
		void init() {};
		const std::string &getMusicFileName() { return musicFileName; };
	protected:
		std::string musicFileName;
	};

	class Content_ChangeBG : public Content_Normal {
		DECLARE_COMMONFUNC(ChangeBG);
	public:
		void init() {};
//		const std::string &getMusicFileName() { return musicFileName; };
	protected:
		// 
//		std::string musicFileName;
	};

	class Content_PlaySE : public Content_Normal {
		DECLARE_COMMONFUNC(PlaySE);
	public:
		void init() {};
		const std::string &getSoundFileName() { return soundFileName; };
	protected:
		std::string soundFileName;
	};

	class Content_Wait : public Content_Normal {
		DECLARE_COMMONFUNC(Wait);
	public:
		void init() {};
		int getLength() { return length; };
	protected:
		int length;
	};

	class Content_Effect : public Content_Normal {
		DECLARE_COMMONFUNC(Effect);
	public:
		void init() {};
//		const std::string &getMusicFileName() { return musicFile; };
	protected:
		// 
//		std::vector< shared_ptr<Motion> > motions;
	};

	class Content_Branch_MemberChoice : public Content_Branch {
		DECLARE_COMMONFUNC(Branch_MemberChoice);
	public:
		void init() { includeBads = false; random = false; };
	protected:
		bool includeBads;
		bool random;
	};

	class Content_Branch_AbilityCheck : public Content_Branch {
		DECLARE_COMMONFUNC(Branch_AbilityCheck);
	public:
		void init() { level = target = physicalRequirement = mentalRequirement = 0; };
	protected:
		int level;
		int target;
		int physicalRequirement;
		int mentalRequirement;
	};

	class Content_Branch_Random : public Content_Branch {
		DECLARE_COMMONFUNC(Branch_Random);
	public:
		void init() { possibility = 0; };
		int getPossibitily() { return possibility; };
	protected:
		int possibility;
	};

	class Content_Branch_Flag : public Content_Branch {
		DECLARE_COMMONFUNC(Branch_Flag);
	public:
		void init() {};
		std::string getFlagName() { return flagName; };
	protected:
		std::string flagName;
	};

	class Content_SetFlag : public Content_Normal {
		DECLARE_COMMONFUNC(SetFlag);
	public:
		void init() { value = false; };
		std::string getFlagName() { return flagName; };
		bool getValue() { return value; };
	protected:
		std::string flagName;
		bool value;
	};

	class Content_MultiBranch_Step : public Content_Branch {
		DECLARE_COMMONFUNC(MultiBranch_Step);
	public:
		void init() {};
		std::string getStepName() { return stepName; };
	protected:
		std::string stepName;
	};

	class Content_SetStep : public Content_Normal {
		DECLARE_COMMONFUNC(SetStep);
	public:
		void init() { value = 0; };
		std::string getStepName() { return stepName; };
		int getValue() { return value; }
	protected:
		std::string stepName;
		int value;
	};

	class Content_Branch_Presense_Cast : public Content_Branch {
		DECLARE_COMMONFUNC(Branch_Presense_Cast);
	public:
		void init() { cardNo = 0; };
		int getCardNo() { return cardNo; };
	protected:
		int cardNo;
	};

	class Content_Branch_Posession_Item : public Content_Branch {
		DECLARE_COMMONFUNC(Branch_Posession_Item);
	public:
		void init() { cardNo = count = range = 0; };
		int getCardNo() { return cardNo; };
		int getCount() { return count; };
		int getRange() { return range; };
	protected:
		int cardNo;
		int count;
		int range;
	};

	class Content_Branch_Posession_Skill : public Content_Branch {
		DECLARE_COMMONFUNC(Branch_Posession_Skill);
	public:
		void init() { cardNo = count = range = 0; };
		int getCardNo() { return cardNo; };
		int getCount() { return count; };
		int getRange() { return range; };
	protected:
		int cardNo;
		int count;
		int range;
	};

	class Content_Branch_Posession_Info : public Content_Branch {
		DECLARE_COMMONFUNC(Branch_Posession_Info);
	public:
		void init() { cardNo = 0; };
		int getCardNo() { return cardNo; };
	protected:
		int cardNo;
	};

	class Content_Branch_Presense_Beast : public Content_Branch {
		DECLARE_COMMONFUNC(Branch_Presense_Beast);
	public:
		void init() { cardNo = count = range = 0; };
		int getCardNo() { return cardNo; };
		int getCount() { return count; };
		int getRange() { return range; };
	protected:
		int cardNo;
		int count;
		int range;
	};

	class Content_Branch_Money : public Content_Branch {
		DECLARE_COMMONFUNC(Branch_Money);
	public:
		void init() { value = 0; };
		int getValue() { return value; }
	protected:
		int value;
	};

	class Content_Branch_Posession_Coupon : public Content_Branch {
		DECLARE_COMMONFUNC(Branch_Posession_Coupon);
	public:
		void init() { unknown = range = 0; };
		std::string getCouponName() { return couponName; };
//		int getUnknown() { return unknown; };
		int getRange() { return range; };
	protected:
		std::string couponName;
		int unknown;	// sȈ
		int range;
	};

	class Content_GetCast : public Content_Normal {
		DECLARE_COMMONFUNC(GetCast);
	public:
		void init() { cardNo = 0; };
		int getCardNo() { return cardNo; };
	protected:
		int cardNo;
	};

	class Content_GetItem : public Content_Normal {
		DECLARE_COMMONFUNC(GetItem);
	public:
		void init() { cardNo = count = range = 0; };
		int getCardNo() { return cardNo; };
		int getCount() { return count; };
		int getRange() { return range; };
	protected:
		int cardNo;
		int count;
		int range;
	};

	class Content_GetSkill : public Content_Normal {
		DECLARE_COMMONFUNC(GetSkill);
	public:
		void init() { cardNo = count = range = 0; };
		int getCardNo() { return cardNo; };
		int getCount() { return count; };
		int getRange() { return range; };
	protected:
		int cardNo;
		int count;
		int range;
	};

	class Content_GetInfo : public Content_Normal {
		DECLARE_COMMONFUNC(GetInfo);
	public:
		void init() { cardNo = 0; };
		int getCardNo() { return cardNo; };
	protected:
		int cardNo;
	};

	class Content_GetBeast : public Content_Normal {
		DECLARE_COMMONFUNC(GetBeast);
	public:
		void init() { cardNo = count = range = 0; };
		int getCardNo() { return cardNo; };
		int getCount() { return count; };
		int getRange() { return range; };
	protected:
		int cardNo;
		int count;
		int range;
	};

	class Content_GetMoney : public Content_Normal {
		DECLARE_COMMONFUNC(GetMoney);
	public:
		void init() { value = 0; };
		int getValue() { return value; };
	protected:
		int value;
	};

	class Content_GetCoupon : public Content_Normal {
		DECLARE_COMMONFUNC(GetCoupon);
	public:
		void init() { point = range = 0; };
		std::string getCouponName() { return couponName; };
		int getPoint() { return point; };
		int getRange() { return range; };
	protected:
		std::string couponName;
		int point;
		int range;
	};

	class Content_LoseCast : public Content_Normal {
		DECLARE_COMMONFUNC(LoseCast);
	public:
		void init() { cardNo = 0; };
		int getCardNo() { return cardNo; };
	protected:
		int cardNo;
	};

	class Content_LoseItem : public Content_Normal {
		DECLARE_COMMONFUNC(LoseItem);
	public:
		void init() { cardNo = count = range = 0; };
		int getCardNo() { return cardNo; };
		int getCount() { return count; };
		int getRange() { return range; };
	protected:
		int cardNo;
		int count;
		int range;
	};

	class Content_LoseSkill : public Content_Normal {
		DECLARE_COMMONFUNC(LoseSkill);
	public:
		void init() { cardNo = count = range = 0; };
		int getCardNo() { return cardNo; };
		int getCount() { return count; };
		int getRange() { return range; };
	protected:
		int cardNo;
		int count;
		int range;
	};

	class Content_LoseInfo : public Content_Normal {
		DECLARE_COMMONFUNC(LoseInfo);
	public:
		void init() { cardNo = 0; };
		int getCardNo() { return cardNo; };
	protected:
		int cardNo;
	};

	class Content_LoseBeast : public Content_Normal {
		DECLARE_COMMONFUNC(LoseBeast);
	public:
		void init() { cardNo = count = range = 0; };
		int getCardNo() { return cardNo; };
		int getCount() { return count; };
		int getRange() { return range; };
	protected:
		int cardNo;
		int count;
		int range;
	};

	class Content_LoseMoney : public Content_Normal {
		DECLARE_COMMONFUNC(LoseMoney);
	public:
		void init() { value = 0; };
		int getValue() { return value; };
	protected:
		int value;
	};

	class Content_LoseCoupon : public Content_Normal {
		DECLARE_COMMONFUNC(LoseCoupon);
	public:
		void init() { range = 0; };
		std::string getCouponName() { return couponName; };
		int getRange() { return range; };
	protected:
		std::string couponName;
		int range;
	};

	class Content_Serifu : public Content_Normal {
		DECLARE_COMMONFUNC(Serifu);
	public:
		void init() {};
	protected:
	};

	class Content_IncStep : public Content_Normal {
		DECLARE_COMMONFUNC(IncStep);
	public:
		void init() {};
		std::string getStepName() { return stepName; };
	protected:
		std::string stepName;
	};

	class Content_DecStep : public Content_Normal {
		DECLARE_COMMONFUNC(DecStep);
	public:
		void init() {};
		std::string getStepName() { return stepName; };
	protected:
		std::string stepName;
	};

	class Content_InvertFlag : public Content_Normal {
		DECLARE_COMMONFUNC(InvertFlag);
	public:
		void init() {};
		std::string getFlagName() { return flagName; };
	protected:
		std::string flagName;
	};

	class Content_BiBranch_Step : public Content_Branch {
		DECLARE_COMMONFUNC(BiBranch_Step);
	public:
		void init() { value = 0; };
		std::string getStepName() { return stepName; };
	protected:
		std::string stepName;
		int value;
	};

	class Content_IncRound : public Content_Normal {
		DECLARE_COMMONFUNC(IncRound);
	public:
		void init() {};
	protected:
	};

	class Content_Branch_Level : public Content_Branch {
		DECLARE_COMMONFUNC(Branch_Level);
	public:
		void init() { isAverage = false; value = 0; };
	protected:
		bool isAverage;
		int value;
	};

	class Content_Branch_Status : public Content_Branch {
		DECLARE_COMMONFUNC(Branch_Status);
	public:
		void init() { status = target = 0; };
		int getStatus() { return status; };
		int getTarget() { return target; };
	protected:
		int status;
		int target;
	};

	class Content_Branch_MemberCount : public Content_Branch {
		DECLARE_COMMONFUNC(Branch_MemberCount);
	public:
		void init() { count = 0; };
		int getCount() { return count; };
	protected:
		int count;
	};

	class Content_ShowParty : public Content_Normal {
		DECLARE_COMMONFUNC(ShowParty);
	public:
		void init() {};
	protected:
	};

	class Content_HideParty : public Content_Normal {
		DECLARE_COMMONFUNC(HideParty);
	public:
		void init() {};
	protected:
	};

	class Content_BreakEffect : public Content_Terminal {
		DECLARE_COMMONFUNC(BreakEffect);
	public:
		void init() {};
	protected:
	};

	class Content_CallStart : public Content_Normal {
		DECLARE_COMMONFUNC(CallStart);
	public:
		void init() {};
		std::string getStartName() { return startName; };
	protected:
		std::string startName;
	};

	class Content_LinkToPackage : public Content_Terminal {
		DECLARE_COMMONFUNC(LinkToPackage);
	public:
		void init() { packageNo = 0; };
		int getPackageNo() { return packageNo; };
	protected:
		int packageNo;
	};

	class Content_CallPackage : public Content_Normal {
		DECLARE_COMMONFUNC(CallPackage);
	public:
		void init() { packageNo = 0; };
		int getPackageNo() { return packageNo; };
	protected:
		int packageNo;
	};

	class Content_Branch_Area : public Content_Branch {
		DECLARE_COMMONFUNC(Branch_Area);
	public:
		void init() {};
	protected:
	};

	class Content_Branch_Battle : public Content_Branch {
		DECLARE_COMMONFUNC(Branch_Battle);
	public:
		void init() {};
	protected:
	};

	class Content_Branch_ClearedScenario : public Content_Branch {
		DECLARE_COMMONFUNC(Branch_ClearedScenario);
	public:
		void init() {};
		std::string getScenarioName() { return scenarioName; };
	protected:
		std::string scenarioName;
	};

	class Content_AddClearedScenario : public Content_Normal {
		DECLARE_COMMONFUNC(AddClearedScenario);
	public:
		void init() {};
		std::string getScenarioName() { return scenarioName; };
	protected:
		std::string scenarioName;
	};

	class Content_RemoveClearedScenaio : public Content_Normal {
		DECLARE_COMMONFUNC(RemoveClearedScenaio);
	public:
		void init() {};
		std::string getScenarioName() { return scenarioName; };
	protected:
		std::string scenarioName;
	};

	class Content_Branch_Gossip : public Content_Branch {
		DECLARE_COMMONFUNC(Branch_Gossip);
	public:
		void init() {};
		std::string getGossipName() { return gossipName; };
	protected:
		std::string gossipName;
	};

	class Content_AddGossip : public Content_Normal {
		DECLARE_COMMONFUNC(AddGossip);
	public:
		void init() {};
		std::string getGossipName() { return gossipName; };
	protected:
		std::string gossipName;
	};

	class Content_RemoveGossip : public Content_Normal {
		DECLARE_COMMONFUNC(RemoveGossip);
	public:
		void init() {};
		std::string getGossipName() { return gossipName; };
	protected:
		std::string gossipName;
	};

	class Content_Branch_BattleResult : public Content_Branch {
		DECLARE_COMMONFUNC(Branch_BattleResult);
	public:
		void init() {};
	protected:
	};

	class Content_RedrawScreen : public Content_Normal {
		DECLARE_COMMONFUNC(RedrawScreen);
	public:
		void init() {};
	protected:
	};

	class Content_FlagCheck : public Content_Normal {
		DECLARE_COMMONFUNC(FlagCheck);
	public:
		void init() {};
		std::string getFlagName() { return flagName; };
	protected:
		std::string flagName;
	};


#undef DECLARE_COMMONFUNC
}

#endif // CARDWIRTH_CONTENT_H_INCLUDED
